#include <QBuffer>
#include <QImageWriter>
#include "ImageWindow.hxx"
#include "ui_ImageWindow.h"

// --- constructors and deconstructors ---

ImageWindow::ImageWindow(QWidget *parent)
: QMainWindow(parent), _ui(new Ui::ImageWindow), _num(0)
{
    _ui->setupUi(this);

    setupActions();
    setMode(Mode::Fresh);
}

ImageWindow::~ImageWindow()
{
    delete _ui;
}

// --- public slots ---

void ImageWindow::slotImageSave()
{
    QBuffer buffer;
    QImageWriter image(&buffer, "BMP");

    image.write(_ui->lab_image->pixmap()->toImage());

    _entry.setOData(buffer.buffer());
    _entry.setOSize(buffer.size());
    _entry.setCSize(_entry.cData().size());
    _entry.setName(_ui->ed_name->text());
    _entry.setNameLength(_entry.name().size() + 1);
    _entry.setHSize(_entry.headerSize() + _entry.nameLength());

    emit sigImageUpdate(_entry, _num);
}

void ImageWindow::slotImageDiscard()
{
    _entry = NCFile();
    _num = 0;
    _ui->lab_image->clear();

    emit sigImageIgnored();
}

void ImageWindow::slotImageView(NCFile& entry, quint32 row)
{
    _entry = entry;
    _num = row;
    _ui->lab_image->setPixmap(QPixmap::fromImage(QImage::fromData(entry.oData())));
    _ui->ed_name->setText(entry.name());
    _ui->ed_size_width->setValue(_ui->lab_image->pixmap()->width());
    _ui->ed_size_height->setValue(_ui->lab_image->pixmap()->height());
}

// --- protected methods ---

void ImageWindow::changeEvent(QEvent *event)
{
    if (event->type() == QEvent::LanguageChange)
        _ui->retranslateUi(this);
    else
        QMainWindow::changeEvent(event);
}

void ImageWindow::setMode(const Mode mode)
{
    switch (mode)
    {
        case Mode::Edited:
            _ui->act_image_save->setEnabled(true);

            break;

        case Mode::Fresh:
        default:
            _ui->act_image_save->setEnabled(false);
    }
}

void ImageWindow::setupActions()
{
    connect(_ui->act_image_save, SIGNAL(triggered()),
            this, SLOT(slotImageSave()));
    connect(_ui->act_image_cancel, SIGNAL(triggered()),
            this, SLOT(slotImageDiscard()));
}
