#pragma once

#include <QMainWindow>
#include "common/NCTypes.hxx"

namespace Ui
{
    class MainWindow;
}
class QLabel;
class QSettings;
class QTranslator;
class NCFile;
class NCFilesystem;
class ImageWindow;
class TextWindow;
class DatWindow;
class ObjectWindow;

class MainWindow : public QMainWindow {
    Q_OBJECT
public:
    // internal typedefs
    using ColType = NC::FileData::Type;

    enum class Language : quint8 {
        English,
        French,
        German
    };

    enum class Mode : quint8 {
        Fresh,
        Edited,
        Selected,
        Blocked
    };

    // constructors and deconstructors
    explicit MainWindow(QWidget *parent = 0);
    virtual ~MainWindow();

public slots:
    // public slots
    void slotAppNew();
    void slotAppOpen();
    void slotAppSave();
    void slotAppSaveAs();
    void slotAppQuit();

    void slotFileAdd();
    void slotFileReplace();
    void slotFileRemove();
    void slotFileSavePlain();
    void slotFileSavePAK();
    void slotFileSaveZLIB();
    void slotFileConvWin();
    void slotFileConvLin();
    void slotFileImage();
    void slotFileText();
    void slotFileDat();
    void slotFileObject();

    void slotLangEnglish();
    void slotLangFrench();
    void slotLangGerman();

    void slotStyleDefault();
    void slotStyleFusion();
    void slotStyleWindows();
    void slotStyleGTK();

    void slotAboutApp();
    void slotAboutQt();

    void slotStatusUpdate();

    void slotTableSelected();
    void slotTableChange(qint32 row);
    void slotTableStatus(quint32 from_row, quint32 to_row, ColType cols);
    void slotTableAdd(NCFile& entry, quint32 row);
    void slotTableReplace(NCFile& entry, quint32 row);
    void slotTableRemove(quint32 row);

    void slotEntryReplace(NCFile& entry, quint32 row);
    void slotEntryIgnore();

protected:
    // protected stuff
    virtual void changeEvent(QEvent *event);
    void setLanguage(const Language lang = Language::English);
    void setMode(const Mode mode = Mode::Fresh);
    void setupActions();

private:
    Ui::MainWindow *_ui;
    QTranslator *_translator;
    QSettings *_settings;
    NCFilesystem *_vfs;
    ImageWindow *_img;
    TextWindow *_txt;
    DatWindow *_dat;
    ObjectWindow *_obj;
    QLabel *_def_status;
    QString _filename;
};
