#include <QString>
#include "TextWindow.hxx"
#include "ui_TextWindow.h"
#include "DefSyntax.hxx"
#include "FxSyntax.hxx"
#include "IniSyntax.hxx"
#include "TscSyntax.hxx"

// --- constructors and deconstructors ---

TextWindow::TextWindow(QWidget *parent)
: QMainWindow(parent), _ui(new Ui::TextWindow), _syn(0), _num(0)
{
    _ui->setupUi(this);

    setupActions();
    setMode(Mode::Fresh);
}

TextWindow::~TextWindow()
{
    delete _syn;
    delete _ui;
}

// --- public slots ---

void TextWindow::slotTextSave()
{
    _entry.setOData(_ui->ed_text->toPlainText().toLatin1());
    _entry.setOSize(_ui->ed_text->toPlainText().toLatin1().size());
    _entry.setCSize(_entry.cData().size());
    _entry.setName(_ui->ed_name->text());
    _entry.setNameLength(_entry.name().size() + 1);
    _entry.setHSize(_entry.headerSize() + _entry.nameLength());

    emit sigTextUpdate(_entry, _num);

    setMode(Mode::Fresh);
}

void TextWindow::slotTextDiscard()
{
    _entry = NCFile();
    _num = 0;
    _ui->ed_text->clear();

    emit sigTextIgnored();

    setMode(Mode::Fresh);
}

void TextWindow::slotEditDelete()
{
    _ui->ed_text->textCursor().removeSelectedText();
}

void TextWindow::slotTextView(NCFile& entry, quint32 row)
{
    _entry = entry;
    _num = row;

    _ui->ed_text->setPlainText(entry.oData());
    _ui->ed_name->setText(entry.name());
    _ui->ed_lines->setText(QString::number(_ui->ed_text->document()->lineCount()));
    _ui->ed_size->setText(QString::number(_ui->ed_text->toPlainText().size()));

    delete _syn;

    if (entry.name().contains(".def", Qt::CaseInsensitive))
        _syn = new DefSyntax(_ui->ed_text->document());

    if (entry.name().contains(".fx", Qt::CaseInsensitive))
        _syn = new FxSyntax(_ui->ed_text->document());

    if (entry.name().contains(".ini", Qt::CaseInsensitive))
        _syn = new IniSyntax(_ui->ed_text->document());

    if (entry.name().contains(".tsc", Qt::CaseInsensitive))
        _syn = new TscSyntax(_ui->ed_text->document());
}

void TextWindow::slotTextChanged()
{
    setMode(Mode::Edited);
}

// --- protected methods ---

void TextWindow::changeEvent(QEvent *event)
{
    if (event->type() == QEvent::LanguageChange)
        _ui->retranslateUi(this);
    else
        QMainWindow::changeEvent(event);
}

void TextWindow::setMode(const Mode mode)
{
    switch (mode)
    {
        case Mode::Edited:
            _ui->act_text_save->setEnabled(true);

            break;

        case Mode::Fresh:
        default:
            _ui->act_text_save->setEnabled(false);
    }
}

void TextWindow::setupActions()
{
    QAction *ctx_sep1 = new QAction(this);
    QAction *ctx_sep2 = new QAction(this);

    // file stuff
    connect(_ui->act_text_save, SIGNAL(triggered()),
            this, SLOT(slotTextSave()));
    connect(_ui->act_text_cancel, SIGNAL(triggered()),
            this, SLOT(slotTextDiscard()));

    // edit stuff
    connect(_ui->act_edit_undo, SIGNAL(triggered()),
            _ui->ed_text, SLOT(undo()));
    connect(_ui->act_edit_redo, SIGNAL(triggered()),
            _ui->ed_text, SLOT(redo()));
    connect(_ui->act_edit_cut, SIGNAL(triggered()),
            _ui->ed_text, SLOT(cut()));
    connect(_ui->act_edit_copy, SIGNAL(triggered()),
            _ui->ed_text, SLOT(copy()));
    connect(_ui->act_edit_insert, SIGNAL(triggered()),
            _ui->ed_text, SLOT(paste()));
    connect(_ui->act_edit_delete, SIGNAL(triggered()),
            this, SLOT(slotEditDelete()));
    connect(_ui->act_edit_selall, SIGNAL(triggered()),
            _ui->ed_text, SLOT(selectAll()));

    // recognize changes
    connect(_ui->ed_text, SIGNAL(textChanged()),
            this, SLOT(slotTextChanged()));

    // context menu
    ctx_sep1->setSeparator(true);
    ctx_sep2->setSeparator(true);

    _ui->ed_text->addAction(_ui->act_edit_undo);
    _ui->ed_text->addAction(_ui->act_edit_redo);
    _ui->ed_text->addAction(ctx_sep1);
    _ui->ed_text->addAction(_ui->act_edit_cut);
    _ui->ed_text->addAction(_ui->act_edit_copy);
    _ui->ed_text->addAction(_ui->act_edit_insert);
    _ui->ed_text->addAction(_ui->act_edit_delete);
    _ui->ed_text->addAction(_ui->act_edit_selall);
    _ui->ed_text->addAction(ctx_sep2);
    _ui->ed_text->addAction(_ui->act_text_save);
    _ui->ed_text->addAction(_ui->act_text_cancel);
}
